@@ -21,16 +21,20 @@ provided here.
 import simplejson
 
 import zope.location
+from zope.container.interfaces import IReadContainer
+import zope.errorview.browser
 from zope import component
 from zope import interface
-from zope.securitypolicy.role import Role as securitypolicy_Role
+from zope.interface.common.interfaces import IException
 from zope.publisher.browser import BrowserPage
-from zope.publisher.interfaces import NotFound
+from zope.publisher.defaultview import getDefaultViewName
 from zope.publisher.interfaces.browser import IBrowserPublisher
 from zope.publisher.interfaces.http import IHTTPRequest
+from zope.publisher.interfaces import INotFound
+from zope.publisher.interfaces import NotFound
 from zope.publisher.publish import mapply
-from zope.publisher.defaultview import getDefaultViewName
-from zope.container.interfaces import IReadContainer
+from zope.security.interfaces import IUnauthorized
+from zope.securitypolicy.role import Role as securitypolicy_Role
 
 import grok
 import martian.util
@@ -41,6 +45,8 @@ import grokcore.message
 from grok import interfaces, util
 
 # BBB this is for import backward compatibility.
+from grokcore.xmlrpc import XMLRPC
+from grokcore.rest import REST
 from grokcore.json import JSON
 from grokcore.content import Model, Container, OrderedContainer
 
@@ -48,20 +54,24 @@ from grokcore.content import Model, Container, OrderedContainer
 class Application(grokcore.site.Site):
     """Mixin for creating Grok application objects.
 
-    When a `grok.Container` (or a `grok.Model`, though most developers
-    use containers) also inherits from `grok.Application`, it not only
-    gains the component registration abilities of a `grok.Site`, but
-    will also be listed in the Grok admin control panel as one of the
-    applications that the admin can install directly at the root of
-    their Zope database.
+    When a :class:`grok.Container` (or a :class:`grok.Model`, though
+    most developers use containers) also inherits from
+    :class:`grok.Application`, it not only gains the component
+    registration abilities of a :class:`grok.Site`, but will also be
+    listed in the Grok admin control panel as one of the applications
+    that the admin can install directly at the root of their Zope
+    database.
 
     """
-    interface.implements(interfaces.IApplication)
+    interface.implements(grokcore.site.interfaces.IApplication)
 
 
 class View(grokcore.view.View):
     """The base class for views with templates in Grok applications.
 
+    Implements the :class:`grokcore.view.interfaces.IGrokView`
+    interface.
+
     Each class that inherits from `grok.View` is designed to "render" a
     category of content objects by reducing them to a document (often an
     HTML document).  Every view has a name, and is invoked when users
@@ -132,7 +142,10 @@ class View(grokcore.view.View):
     interface.implements(interfaces.IGrokView)
 
     def application_url(self, name=None, data=None):
-        """Return the URL of the nearest enclosing `grok.Application`."""
+        """Return the URL of the closest :class:`grok.Application` object in
+        the hierarchy or the URL of a named object (``name``
+        parameter) relative to the closest application object.
+        """
         return util.application_url(self.request, self.context, name, data)
 
     def flash(self, message, type='message'):
@@ -140,17 +153,94 @@ class View(grokcore.view.View):
         grokcore.message.send(message, type=type, name='session')
 
 
+class ExceptionView(View, zope.errorview.browser.ExceptionView):
+    """Base class for rendering views for uncaught exceptions that occur during
+    the application run-time and are not otherwise rendered.
+
+    Note that when this class in not subclassed, the default error view
+    from zope.errorview is being rendered.
+    """
+    grok.context(IException)
+    grok.name('index.html')
+
+    def update(self):
+        return zope.errorview.browser.ExceptionView.update(self)
+
+    def render(self):
+        """An error view can either be rendered by an associated template, or
+        it can implement this method to render itself from Python. This is
+        useful if the view's output isn't XML/HTML but something computed in
+        Python (plain text, PDF, etc.)
+
+        Contrary to regular views, render() does *not* accept any parameters.
+        """
+        return zope.errorview.browser.ExceptionView.render(self)
+
+    render.base_method = True
+
+
+class NotFoundView(View, zope.errorview.browser.NotFoundView):
+    """Base class for rendering views for INotFound exceptions.
+
+    Note that when this class in not subclassed, the default error view
+    from zope.errorview is being rendered.
+    """
+    grok.context(INotFound)
+    grok.name('index.html')
+
+    def update(self):
+        return zope.errorview.browser.NotFoundView.update(self)
+
+    def render(self):
+        """An error view can either be rendered by an associated template, or
+        it can implement this method to render itself from Python. This is
+        useful if the view's output isn't XML/HTML but something computed in
+        Python (plain text, PDF, etc.)
+
+        Contrary to regular views, render() does *not* accept any parameters.
+        """
+        return zope.errorview.browser.NotFoundView.render(self)
+
+    render.base_method = True
+
+
+class UnauthorizedView(View, zope.errorview.browser.UnauthorizedView):
+    """Base class for rendering views for IUnauthorized exceptions.
+
+    Note that when this class in not subclassed, the default error view
+    from zope.errorview is being rendered.
+    """
+    grok.context(IUnauthorized)
+    grok.name('index.html')
+
+    def update(self):
+        return zope.errorview.browser.UnauthorizedView.update(self)
+
+    def render(self):
+        """An error view can either be rendered by an associated template, or
+        it can implement this method to render itself from Python. This is
+        useful if the view's output isn't XML/HTML but something computed in
+        Python (plain text, PDF, etc.)
+
+        Contrary to regular views, render() does *not* accept any parameters.
+        """
+        return zope.errorview.browser.UnauthorizedView.render(self)
+
+    render.base_method = True
+
+
 class Form(grokcore.formlib.Form, View):
     """The base class for forms in Grok applications.
 
-    A class that inherits from `grok.Form` is a `grok.View` whose
-    template will be given information about the fields in its context,
-    and use that information to render an HTML form for adding or
-    editing the form.  Generally developers use one of the subclasses:
+    A class that inherits from :class:`grok.Form` is a
+    :class:`grok.View` whose template will be given information about
+    the fields in its context, and use that information to render an
+    HTML form for adding or editing the form.  Generally developers
+    use one of the subclasses:
 
-    * `grok.AddForm`
-    * `grok.DisplayForm`
-    * `grok.EditForm`
+    * :class:`grok.AddForm`
+    * :class:`grok.DisplayForm`
+    * :class:`grok.EditForm`
 
     """
     interface.implements(interfaces.IGrokForm)
@@ -177,146 +267,20 @@ class ViewishViewSupport(grokcore.view.ViewSupport):
         return util.application_url(self.request, self.context, name, data)
 
 
-class XMLRPC(ViewishViewSupport):
-    """Base class for XML-RPC endpoints in Grok applications.
-
-    When an application creates a subclass of `grok.XMLRPC`, it is
-    creating an XML-RPC view.  Like other Grok views, each `grok.XMLRPC`
-    component can either use an explicit `grok.context()` directive to
-    specify the kind of object it wraps, or else Grok will look through
-    the same module for exactly one `grok.Model` or `grok.Container` (or
-    other `IGrokContext` implementor) and make that class its context
-    instead.
-
-    Every object that is an instance of the wrapped class or interface
-    becomes a legitimate XML-RPC server URL, offering as available
-    procedures whatever methods have been defined inside of that
-    `grok.XMLRPC` component.  When a method is called over XML-RPC, any
-    parameters are translated into normal Python data types and supplied
-    as normal positional arguments.  When the method returns a value or
-    raises an exception, the result is converted back into an XML-RPC
-    response for the client.  In both directions, values are marshalled
-    transparently to and from XML-RPC data structures.
-
-    During the execution of an XML-RPC method, the object whose URL was
-    used for the XML-RPC call is available as ``self.context``.
-
-    """
-
-
-class REST(zope.location.Location, ViewishViewSupport):
-    """Base class for REST views in Grok applications."""
-    interface.implements(interfaces.IREST)
-
-    def __init__(self, context, request):
-        self.context = self.__parent__ = context
-        self.request = request
-
-
-class Traverser(object):
-    """Base class for traversers in Grok applications."""
-    interface.implements(IBrowserPublisher)
-
-    def __init__(self, context, request):
-        self.context = context
-        self.request = request
-
-    def browserDefault(self, request):
-        # if we have a RESTful request, we will handle
-        # GET, POST and HEAD differently (PUT and DELETE are handled already
-        # but not on the BrowserRequest layer but the HTTPRequest layer)
-        if interfaces.IRESTLayer.providedBy(request):
-            rest_view = component.getMultiAdapter(
-                (self.context, self.request), name=request.method)
-            return rest_view, ()
-        view_name = getDefaultViewName(self.context, request)
-        view_uri = "@@%s" % view_name
-        return self.context, (view_uri,)
-
-    def publishTraverse(self, request, name):
-        subob = self.traverse(name)
-        if subob is not None:
-            return util.safely_locate_maybe(subob, self.context, name)
-
-        traversable_dict = grok.traversable.bind().get(self.context)
-        if traversable_dict:
-            if name in traversable_dict:
-                subob = getattr(self.context, traversable_dict[name])
-                if callable(subob):
-                    subob = subob()
-                return util.safely_locate_maybe(subob, self.context, name)
-
-        # XXX Special logic here to deal with containers.  It would be
-        # good if we wouldn't have to do this here. One solution is to
-        # rip this out and make you subclass ContainerTraverser if you
-        # wanted to override the traversal behaviour of containers.
-        if IReadContainer.providedBy(self.context):
-            item = self.context.get(name)
-            if item is not None:
-                return item
-
-        view = component.queryMultiAdapter((self.context, request), name=name)
-        if view is not None:
-            return view
-
-        raise NotFound(self.context, name, request)
-
-    def traverse(self, name):
-        # this will be overridden by subclasses
-        pass
-
-
-class ContextTraverser(Traverser):
-    """Base class for context traversers in Grok applications.
-
-    A context traverser is like a normal `grok.Traverser` but, instead
-    of supplying its own `traverse()` method, it directs Grok to go call
-    the ``traverse()`` method on the context itself in order to process
-    the next name in the URL.
-
-    """
-    component.adapts(interfaces.IContext, IHTTPRequest)
-
-    def traverse(self, name):
-        traverse = getattr(self.context, 'traverse', None)
-        if traverse:
-            return traverse(name)
-
-
-class ContainerTraverser(Traverser):
-    """Base class for container traversers in Grok applications.
-
-    A container traverser is like a normal `grok.Traverser` but, instead
-    of supplying its own ``traverse()`` method, Grok will either call
-    the ``traverse()`` method on the context itself, if any, else call
-    ``get()`` on the container (a getitem-style lookup) in order to
-    resolve the next name in the URL.
-
-    """
-    component.adapts(interfaces.IContainer, IHTTPRequest)
-
-    def traverse(self, name):
-        traverse = getattr(self.context, 'traverse', None)
-        if traverse:
-            result = traverse(name)
-            if result is not None:
-                return result
-        # try to get the item from the container
-        return self.context.get(name)
-
-
 class IndexesClass(object):
     """Base class for index collections in a Grok application.
 
-    A `grok.Indexes` utility provides one or more Zope Database content
-    indexes for use in a `grok.Site` or `grok.Application`.  The site or
-    application that the indexes are intended for should be named with
-    the `grok.site()` directive, and the kind of object to index should
-    be named with a `grok.context()` directive.
+    A `grok.Indexes` utility provides one or more Zope Database
+    content indexes for use in a :class:`grok.Site` or
+    :class:`grok.Application`.  The site or application that the
+    indexes are intended for should be named with the :func:`grok.site()`
+    directive, and the kind of object to index should be named with a
+    :func:`grok.context()` directive.
 
     Inside their class, the developer should specify one or more
-    `grok.index.Field` instances naming object attributes that should be
-    indexed (and therefore searchable)::
+    :class:`grok.index.Field`, :class:`grok.index.Text`, or
+    :class:`grok.index.Set` instances naming object attributes that
+    should be indexed (and therefore searchable).::
 
         class ArticleIndex(grok.Indexes):
             grok.site(Newspaper)
@@ -325,15 +289,18 @@ class IndexesClass(object):
             title = index.Field()
             body = index.Text()
 
-    See the `grok.index` module for more information on field types.
+    See the :mod:`grok.index` module for more information on field
+    types.
 
-    Note that indexes are persistent: they are stored in the Zope
-    database alongside the site or application that they index.  They
-    are created when the site or application is first created, and so an
-    already-created site will not change just because the definition of
-    one of its `grok.Indexes` changes; it will either have to be deleted
-    and re-created, or some other operation performed to bring its
-    indexes up to date.
+    .. note:: Indexes are persistent: they are stored in the Zope
+              database alongside the site or application that they
+              index.  They are created when the site or application is
+              first created (and made persistent), and so an
+              already-created site will not change just because the
+              definition of one of its :data:`grok.Indexes` changes;
+              it will either have to be deleted and re-created, or
+              some other operation performed to bring its indexes up
+              to date.
 
     """
     def __init__(self, name, bases=(), attrs=None):
